;;                                                  -*- lexical-binding: t -*-

(require 'profiler)
(require 'cl-lib)

(defun foo ()
  (let ((start (float-time)))
    (while (< (- (float-time) start) 1)
      ))
  nil)

(defun bar ()
  (animate-birthday-present "----"))

(defun record-samples (sampling-interval fun)
  (profiler-cpu-start sampling-interval)
  (funcall fun)
  (profiler-cpu-stop)
  (profiler-cpu-log))

(defun print-calltree (tree depth)
  (princ (format "%8d %4s %s%s\n"
		 (profiler-calltree-count tree)
		 (profiler-calltree-count-percent tree)
		 (make-string (* depth 2) ?\ )
		 (profiler-calltree-entry tree)))
  (let ((children (profiler-calltree-children tree)))
    (dolist (child children)
      (print-calltree child (1+ depth)))))

(defun save-samples (log filename)
  (with-temp-file filename
    (let (print-level print-length)
      (print log (current-buffer)))))

(defun load-samples (filename)
  (with-temp-buffer
    (insert-file-contents filename)
    (goto-char (point-min))
    (read (current-buffer))))

(defun usage ()
  (princ "Usage: emacs -batch -l profler-test -f main CMD
where CMD is one of
  record FUNCTION FILENAME    -- call FUNCTION and write samples to FILENAME
  report FILENAME             -- print call tree for samples in FILENAME
"))

(defun main ()
  (pcase-exhaustive command-line-args-left
    (`("record" ,function-name ,filename)
     (let ((log (record-samples 500000 (intern function-name))))
       (save-samples log filename)))
    (`("report" ,filename)
     (let* ((log (load-samples filename))
	    (calltree (profiler-calltree-build log)))
       (print-calltree calltree 0)))
    (_
     (usage))))
